#include <cstdio>

__global__ void add(float *a, float *b)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    b[i] = a[i] + b[i];
}

int main()
{
    constexpr size_t n = 1'000'000;
    float *a = new float[n], *b = new float[n];
    for (int i = 0; i < n; i++) {
        a[i] = 2.;
        b[i] = 3.;
    }

    float *a_dev, *b_dev;
    cudaMalloc((void**)&a_dev, n*sizeof(float));
    cudaMalloc((void**)&b_dev, n*sizeof(float));
    cudaMemcpy(a_dev, a, n*sizeof(float), cudaMemcpyHostToDevice);
    cudaMemcpy(b_dev, b, n*sizeof(float), cudaMemcpyHostToDevice);

    add<<<n/64,64>>>(a_dev, b_dev);

    cudaMemcpy(b, b_dev, n*sizeof(float), cudaMemcpyDeviceToHost);

    for (int i = 0; i < 10; i++) printf("b[%d] = %f\n", i, b[i]);

    delete[] a;
    delete[] b;
    cudaFree(a_dev);
    cudaFree(b_dev);
}
