#include <cstdio>
#include <thrust/host_vector.h>
#include <thrust/device_vector.h>
#include <thrust/fill.h>

__global__ void add(float *a, float *b)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    b[i] = a[i] + b[i];
}

int main()
{
    constexpr size_t n = 1000000;
    thrust::host_vector<float> a(n), b(n);
    thrust::fill(begin(a), end(a), 2.);
    thrust::fill(begin(b), end(b), 3.);

    thrust::device_vector<float> a_dev = a, b_dev = b;

    float *a_dev_ptr = thrust::raw_pointer_cast(a_dev.data());
    float *b_dev_ptr = thrust::raw_pointer_cast(b_dev.data());
    add<<<n/64,64>>>(a_dev_ptr, b_dev_ptr);

    b = b_dev;

    for (int i = 0; i < 10; i++) printf("b[%d] = %f\n", i, b[i]);
}
