#include "hip/hip_runtime.h"
#include <cstdio>

__global__ void add(float *a, float *b)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    b[i] = a[i] + b[i];
}

int main()
{
    constexpr size_t n = 1000000;
    float a[n], b[n];
    for (int i = 0; i < n; i++) {
        a[i] = 2.;
        b[i] = 3.;
    }

    float *a_dev, *b_dev;
    hipMalloc((void **)&a_dev, n*sizeof(float));
    hipMalloc((void **)&b_dev, n*sizeof(float));
    hipMemcpy(a_dev, a, n*sizeof(float), hipMemcpyHostToDevice);
    hipMemcpy(b_dev, b, n*sizeof(float), hipMemcpyHostToDevice);

    hipLaunchKernelGGL(add, dim3(n/64), dim3(64), 0, 0, a_dev, b_dev);

    hipMemcpy(b, b_dev, n*sizeof(float), hipMemcpyDeviceToHost);

    hipFree(a_dev);
    hipFree(b_dev);

    for (int i = 0; i < 10; i++) printf("b[%d] = %f\n", i, b[i]);
}
