#include <cstdio>
#include <CL/opencl.h>

const char *kernel_source =
"__kernel void add(__global float *a, __global float *b) \n"
"{                                                       \n"
"    int i = get_global_id(0);                           \n"
"    b[i] = a[i] + b[i];                                 \n"
"}                                                       \n";

int main()
{
    constexpr size_t n = 1000000;
    float *a = new float[n], *b = new float[n];
    for (int i = 0; i < n; i++) {
        a[i] = 2.;
        b[i] = 3.;
    }

    cl_platform_id platform;
    clGetPlatformIDs(1, &platform, NULL);
    cl_device_id device_id;
    clGetDeviceIDs(platform, CL_DEVICE_TYPE_GPU, 1, &device_id, NULL);
    cl_int err;
    cl_context context = clCreateContext(0, 1, &device_id, NULL, NULL, &err);
    cl_command_queue queue = clCreateCommandQueue(context, device_id, 0, &err);

    cl_mem a_dev;
    cl_mem b_dev;
    a_dev = clCreateBuffer(context, CL_MEM_READ_ONLY,  n*sizeof(float), NULL, NULL);
    b_dev = clCreateBuffer(context, CL_MEM_READ_WRITE, n*sizeof(float), NULL, NULL);
    clEnqueueWriteBuffer(queue, a_dev, CL_TRUE, 0, n*sizeof(float), a, 0, NULL, NULL);
    clEnqueueWriteBuffer(queue, b_dev, CL_TRUE, 0, n*sizeof(float), b, 0, NULL, NULL);

    cl_program program = clCreateProgramWithSource(context, 1, &kernel_source, NULL, &err);
    clBuildProgram(program, 0, NULL, NULL, NULL, NULL);
    cl_kernel kernel = clCreateKernel(program, "add", &err);
    clSetKernelArg(kernel, 0, sizeof(cl_mem), &a_dev);
    clSetKernelArg(kernel, 1, sizeof(cl_mem), &b_dev);
    size_t global_size = n, local_size = 64;
    clEnqueueNDRangeKernel(queue, kernel, 1, NULL, &global_size, &local_size, 0, NULL, NULL);

    clFinish(queue);
    clEnqueueReadBuffer(queue, b_dev, CL_TRUE, 0, n*sizeof(float), b, 0, NULL, NULL);

    for (int i = 0; i < 10; i++) printf("b[%d] = %f\n", i, b[i]);

    delete[] a;
    delete[] b;
    clReleaseMemObject(a_dev);
    clReleaseMemObject(b_dev);
    clReleaseProgram(program);
    clReleaseKernel(kernel);
    clReleaseCommandQueue(queue);
    clReleaseContext(context);
}
